# based off: https://github.com/pysal/esda/blob/master/tests/test_moran.py#L96
import pytest

import libpysal
import numpy as np

from esda.losh import LOSH

from pathlib import Path
examples_stl_gal =  Path("/usr/share/doc/python3-libpysal/examples/stl/stl.gal")
examples_stl_txt = Path("/usr/share/doc/python3-libpysal/examples/stl/stl_hom.txt")

parametrize_w = pytest.mark.parametrize(
    "w",
    [
        libpysal.io.open(examples_stl_gal).read(),
        libpysal.graph.Graph.from_W(
            libpysal.io.open(examples_stl_gal).read()
        ),
    ],
    ids=["W", "Graph"],
)

class TestLosh:
    def setup_method(self):
        np.random.seed(10)
        f = f = libpysal.io.open(examples_stl_txt)
        self.y = np.array(f.by_col["HR8893"])

    @parametrize_w
    def test_losh(self, w):
        ls = LOSH(connectivity=w, inference="chi-square").fit(self.y)
        np.testing.assert_allclose(ls.Hi[0], 0.77613471)
        np.testing.assert_allclose(ls.pval[0], 0.22802201)
